<?php
/**
 * Data structures used in parsing XML DocBook-based package pages
 * @package phpDocumentor
 * @subpackage Tutorial
 */
/**
 * @package phpDocumentor
 * @subpackage Tutorial
 */
class parserCData extends parserStringWithInlineTags
{
    function Convert(&$c)
    {
        foreach($this->value as $key => $value)
        {
            if (is_string($value)) $this->value[$key] = $c->getCData($value);
        }
        return parent::Convert($c, false);
    }
}
/**
 * a standard XML DocBook Tag
 *
 * This class is designed to represent uncomplicated DocBook tags like abbrev
 * and blockquote that have no subtags beyond title
 * @since 1.2
 * @package phpDocumentor
 * @subpackage Tutorial
 */
class parserXMLDocBookTag extends parserStringWithInlineTags
{
    /**
     * Attributes from the XML tag
     *
     * Format: array(attrname => attrvalue, attrname => attrvalue,...)
     * @var array
     */
    var $attributes = array();
    /**
     * Name of the tag
     * @var string
     */
    var $name;
    
    var $_cdata;
    var $_title;
    var $_id;
    var $_description;
    var $_state;
    
    function parserXMLDocBookTag($name)
    {
        $this->name = $name;
    }
    
    /**
     * @uses Converter::TranslateTag()
     */
    function Convert(&$c, $postprocess = true)
    {
        $value = parent::Convert($c, $postprocess);
        foreach($this->attributes as $a => $v)
        {
            $this->attributes[$a] = (is_string($v) ? $v : $v->Convert($c, $postprocess));
        }
        if (isset($this->_title))
        {
            list($this->attributes,$value) = $c->ConvertTitle($this->name, $this->attributes, $this->_title->Convert($c, $postprocess), $value);
        }
        return $c->TranslateTag($this->name,$this->attributes,$value);
    }
    
    function startCData()
    {
        $this->_cdata = new parserCData;
    }
    
    function endCData()
    {
        $this->value[] = $this->_cdata;
        unset($this->_cdata);
    }
    
    function getTOC($state = false)
    {
        if ($state !== false) return $this->value[$state];
        return count($this->value);
    }
    
    function setTOC($state, $val)
    {
        $this->value[$state] = $val;
    }
    
    function addCData($word)
    {
        $this->_cdata->add($word);
    }
    
    function addAttribute($name,$value)
    {
        $this->attributes[$name] = $value;
        if ($name == 'id') $this->setId($value);
    }
    
    function setTitle($title)
    {
        $this->_title = $title;
    }
    
    function setId($id)
    {
        $this->_id = $id;
    }
    
    function getId(&$c)
    {
        if ($this->_id) return trim($this->_id->Convert($c));
    }
    
    function hasTitle()
    {
        return isset($this->_title);
    }
    
    /**
     * @param Converter
     */
    function getTitle(&$c)
    {
        if ($this->name == 'refentry')
        {
            foreach($this->value as $tag)
            {
                if (is_object($tag) && $tag->name == 'refnamediv')
                {
                    return $tag->getTitle($c);
                }
            }
        }
        if ($this->name == 'refnamediv')
        {
            foreach($this->value as $tag)
            {
                if (is_object($tag) && $tag->name == 'refname')
                {
                    $t = new parserStringWithInlineTags;
                    foreach($tag->value as $val) $t->add($val);
                    $this->_title = $t;
                }
                if (is_object($tag) && $tag->name == 'refpurpose')
                {
                    $t = new parserStringWithInlineTags;
                    foreach($tag->value as $val) $t->add($val);
                    $this->_description = $t;
                }
            }
        }
        if (isset($this->_title))
        return $this->_title->Convert($c);
        if (is_object($this->value[0])) return $this->value[0]->getTitle($c);
        if (isset($this->value[1]))
        if (is_object($this->value[1])) return $this->value[1]->getTitle($c);
        return '';
    }
    
    function getSubsection(&$c,$subsection)
    {
        $search = $this->_id;
        if (is_string($this->_id)) return false;
        if (get_class($search) != 'parseridinlinetag') return false;
        $search->id = $subsection;
        foreach($this->value as $el)
        {
            if (get_class($el) == 'parserxmldocbooktag')
            {
                if ($el->getId($c) == $search->Convert($c))
                {
                    return $el;
                } elseif ($a = $el->getSubsection($c,$subsection))
                {
                    return $a;
                }
            }
        }
        return false;
    }
    
    function add($el)
    {
        if (is_string($el)) return parent::add($el);
        if (get_class($el) == 'parserxmldocbooktag')
        {
            if ($el->name == 'title')
            {
                $this->setTitle($el);
            } else return parent::add($el);
        } else return parent::add($el);
    }
}
?>