<?php
/**
 * @author Greg Beaver <cellog@users.sourceforge.net>
 * @since 1.2
 */
class phpDocumentorTWordParser extends WordParser
{
    /**#@+
     * @access private
     */
    /**
     * tokenized array from {@link token_get_all()}
     * @var array
     */
    var $_all;
    /**
     * List of tokens that can contain a newline
     * @var array
     */
    var $_nl_check = array(T_WHITESPACE,T_ENCAPSED_AND_WHITESPACE,T_COMMENT,T_ML_COMMENT,T_OPEN_TAG,T_CLOSE_TAG,T_INLINE_HTML);
    /**
     * @var array
     */
    var $_global_search;
    /**
     * current source line number (relative)
     * @var integer
     */
    var $_sourceline;
    /**
     * Source of the entire file, parsed into arrays of tokens on each line
     * @var array
     */
    var $_file_source = array();
    /**#@-*/
    
    function setup(&$input)
    {
        $this->data = &$input;
        $this->_all = token_get_all($input);
        $this->_file_source = array();
        $this->addFileSource($this->_all);
        $this->_sourceline = 0;
        $this->pos = 0;
        $this->linenum = 0;
    }
    
    function getSource()
    {
        $source = $this->source;
        $this->source = array();
        $this->getsource = false;
        return $source;
    }
    
    function getFileSource()
    {
        return $this->_file_source;
    }
    
    function retrievesource($word = '')
    {
        $this->source = array(array(array(T_FUNCTION,'function'),$word));
        $this->_sourceline = 0;
        $this->getsource = true;
    }

    function tokenEquals($a, $b)
    {
        if (is_array($a)) $a = $a[1];
        if (is_array($b)) $b = $b[1];
        return $a == $b;
    }
    
    function concatTokens($a)
    {
        $b = '';
        foreach($a as $c)
        {
            if (is_array($c)) $c = $c[1];
            $b .= $c;
        }
        return $b;
    }
    
    function getWord()
    {
        if (!isset($this->_all[$this->pos])) return false;
        $word = $this->_all[$this->pos++];
        if (isset($this->_global_search))
        {
            $pos = $this->pos;
            $gpos = 0;
            $found = false;
            if ($this->tokenEquals($word,$this->_global_search[$gpos++]))
            {
                $found = true;
                for(;$gpos<count($this->_global_search);$gpos++,$pos++)
                {
                    if (!$this->tokenEquals($this->_global_search[$gpos],$this->_all[$pos])) $found = false;
                }
            }
            if ($found)
            {
                $a = $this->concatTokens($this->_global_search);
                $this->pos += count($this->_global_search) - 1;
                unset($this->_global_search);
                return $a;
            }
        }
        if ($this->getsource)
        {
            $this->addSource($word);
        }
        if (is_array($word))
        {
            if (in_array($word[0],$this->_nl_check))
            {
                $this->linenum += substr_count($word[1],"\n");
            }
            if ($word[0] == T_WHITESPACE && !$this->returnWhiteSpace) return $this->getWord();
            // seeing if we can get line numbers out of the beast
        }
        return $word;
    }
    
    function addFileSource($word)
    {
        $this->_sourceline = 0;
        foreach($word as $token)
        {
            $this->addSource($token, true);
        }
    }
    
    function addSource($word, $file = false)
    {
        if (is_array($word))
        {
            $lines = explode("\n",$word[1]);
            if (count($lines) > 1)
            {
                for($i = 0;$i<count($lines);$i++)
                {
                    $token = $lines[$i];
                    if ($i < count($lines) - 1) $token = "$token\n";
                    $this->_set_sars($file, array($word[0],$token));
                    if ($i < count($lines) - 1)
                        $this->_sourceline++;
                }
            } else $this->_set_sars($file, $word);
        } else $this->_set_sars($file, $word);
    }
    
    /** @access private */
    function _set_sars($type,$word)
    {
        if ($type)
        {
            $this->_file_source[$this->_sourceline][] = $word;
        } else
        {
            $this->source[$this->_sourceline][] = $word;
        }
    }
    
    function findGlobal($tokens)
    {
        if (!$tokens)
        {
            unset($this->_global_search);
        } else
        $this->_global_search = $tokens;
    }
    
    function backupPos()
    {
        $this->pos--;
        $word = $this->_all[$this->pos];
        if ($this->getsource) unset($this->source[$this->_sourceline][count($this->source[$this->_sourceline]) - 1]);
        if (is_array($word))
        {
            if ($word[0] == T_WHITESPACE && !$this->returnWhiteSpace) return $this->getWord();
            // seeing if we can get line numbers out of the beast
            if (in_array($word[0],$this->_nl_check))
            {
                $this->linenum -= substr_count($word[1],"\n");
            }
        }
    }
}
?>