<refentry id="{@id}">
 <refnamediv>
  <refname>Writing a Converter, Methods</refname>
  <refpurpose>Learn what methods need to be defined in a new Converter</refpurpose>
 </refnamediv>
 <refsynopsisdiv>
  <author>
   Gregory Beaver
   <authorblurb>
    {@link mailto:cellog@sourceforge.com cellog@sourceforge.com}
   </authorblurb>
  </author>
 </refsynopsisdiv>
 <refsect1 id="{@id intro}">
  <title>Introduction to Converter Methods</title>
  <para>This Tutorial describes the methods that a New Converter</para>
 </refsect1>
 <refsect1 id="{@id override}">
  <refsect2 id="{@id convert}">
   <title>Convert()</title>
   <para>The {@link Converter::Convert()} method is called by {@link Converter::walk()} to process all Converter classes:
    <itemizedlist>
     <listitem>{@link parserData} - representation of a file</listitem>
     <listitem>{@link parserInclude} - representation of include statements</listitem>
     <listitem>{@link parserDefine} - representation of define statements</listitem>
     <listitem>{@link parserFunction} - representation of functions</listitem>
     <listitem>{@link parserGlobal} - representation of global variables</listitem>
     <listitem>{@link parserClass} - representation of a class</listitem>
     <listitem>{@link parserVar} - representation of class variables</listitem>
     <listitem>{@link parserMethod} - representation of class methods</listitem>
     <listitem>{@link parserPage} - representation of old Package Pages (deprecated)</listitem>
     <listitem>{@link parserTutorial} - representation of tutorials (like what you are reading now)</listitem>
    </itemizedlist>
    It is up to this method to distribute processing of these elements, or do any post-processing that is necessary.  All of the converters included with phpDocumentor process elements by passing them to individual Convert*() methods like {@link HTMLframesConverter::ConvertClass()}, and one can simply copy this style, or write a completely new method.
   </para>
   <para>Data is passed to the Converter organized by file, procedural elements first followed by class elements, unless {@link Converter::$sort_absolutely_everything} is set to true.  Then data is passed organized by package, with all files and procedural elements first, followed by all class elements.  The PDFdefaultConverter uses $sort_absolutely_everything = true, and HTML converters use it set to false</para>
  </refsect2>
  <refsect2 id="{@id returnsee}">
   <title>returnSee()</title>
   <para>This method takes a {@link abstractLink} class descendant and converts it to an output-format compatible link to an element's documentation.  HTML converters convert the link to an &lt;a href=&quot;&quot;&gt; tag, the XML converter changes it to a &lt;link linkend=&quot;&quot;&gt; tag, etc.  This function is also responsible for distinguishing between sections of documentation.  All information needed to distinguish between elements is included in the data members of a link class, it is up to the returnSee() method to translate that into a unique string identifier.  A good strategy is to write a function that takes a link class and returns a unique identifier as in {@link XMLDocBookConverter::getId()}, and then reference this function to grab identification strings for defining anchors in the generated output.</para>
  </refsect2>
  <refsect2 id="{@id returnlink}">
   <title>returnLink()</title>
   <para>This method takes a URL and converts it to an external link to that URL in the appropriate output format.</para>
  </refsect2>
  <refsect2 id="{@id output}">
   <title>Output()</title>
   <para>This method is called at the end of the walk() method.  It may be used to generate output or do any necessary cleanup.  Nothing is required of Output, and it may do nothing as it does in the HTML converters, which write output continuously as converting.</para>
  </refsect2>
  <refsect2 id="{@id convert-ric}">
   <title>Convert_RIC()</title>
   <para>This method is called to format the contents of README, INSTALL, CHANGELOG, NEWS, and FAQ files grabbed from the base parse directory.  This function allows standard distribution files to be embedded in generated documentation.  A Converter should format these files in a monospace font, if possible.</para>
  </refsect2>
  <refsect2 id="{@id converterrorlog}">
   <title>ConvertErrorLog()</title>
   <para>This method is called at the end of parsing to convert the error log into output for viewing later by the developer.  Error output is very useful for finding mistakes in documentation comments.  A simple solution is to copy the {@link HTMLframesConverter::ConvertErrorLog()} function and the errors.tpl template file to the new converter.  The error log should not be embedded in generated output, as no end-user wants to see that information.</para>
  </refsect2>
  <refsect2 id="{@id formatindex}">
   <title>formatIndex()</title>
   <para>This method is called before processing any elements, and is not required to do anything.  The intent is to allow processing of a global element index to occur in a separate method, which logically separates activities performed by the Converter.  See the {@link HTMLframesConverter::formatIndex()} method for details on one possible implementation</para>
  </refsect2>
  <refsect2 id="{@id formatpkgindex}">
   <title>formatPkgIndex()</title>
   <para>Like formatIndex(), this method is called before processing any elements, and is not required to do anything.  The intent is to allow processing of a package-level index to occur in a separate method, which logically separates activities performed by the Converter.  See the {@link HTMLframesConverter::formatPkgIndex()} method for details on one possible implementation</para>
  </refsect2>
  <refsect2 id="{@id formatleftindex}">
   <title>formatLeftIndex()</title>
   <para>Like formatIndex(), this oddly-named method is called before processing any elements, and is not required to do anything.  The name comes from the original JavaDoc design of putting an index in the lower left frame of output.  The indexes needed by this function are automatically generated based on the value of {@link Converter::$leftindex}.  These indexes are arrays of links organized by package and subpackage.  Left indexes can be generated for files (procedural pages), functions, classes, constants (define statements), and global variables.</para>
  </refsect2>
  <refsect2 id="{@id formattutorialtoc}">
   <title>formatTutorialTOC()</title>
   <para>This method is used to format the automatically generated table of contents from an {@tutorial tags.inlinetoc.pkg} in a tutorial.  The data structure passed is an array generated by {@link parserTocInlineTag::Convert()} that contains each entry.  All that formatTutorialTOC needs to do is structure the inherent hierarchy of the original DocBook tutorial source according to the requirements of the output format.  This is remarkably simple, and can often be implemented simply by passing the entire array to a template engine, as HTMLframesConverter::formatTutorialTOC() does.</para>
  </refsect2>
  <refsect2 id="{@id endpage}">
   <title>endPage()</title>
   <para></para>
  </refsect2>
  <refsect2 id="{@id endclass}">
   <title>endClass()</title>
  </refsect2>
  <refsect2 id="{@id unmangle}">
   <title>unmangle()</title>
  </refsect2>
  <refsect2 id="{@id getfunctionlink}">
   <title>getFunctionLink()</title>
  </refsect2>
  <refsect2 id="{@id getclasslink}">
   <title>getClassLink()</title>
  </refsect2>
  <refsect2 id="{@id getdefinelink}">
   <title>getDefineLink()</title>
  </refsect2>
  <refsect2 id="{@id getgloballink}">
   <title>getGlobalLink()</title>
  </refsect2>
  <refsect2 id="{@id getmethodlink}">
   <title>getMethodLink()</title>
  </refsect2>
  <refsect2 id="{@id getroottree}">
   <title>getRootTree()</title>
  </refsect2>
  <refsect2 id="{@id getvarlink}">
   <title>getVarLink()</title>
  </refsect2>
  <refsect2 id="{@id checkstate}">
   <title>checkState()</title>
  </refsect2>
  <refsect2 id="{@id getstate}">
   <title>getState()</title>
  </refsect2>
  <refsect2 id="{@id type-adjust}">
   <title>type_adjust()</title>
  </refsect2>
  <refsect2 id="{@id postprocess}">
   <title>postProcess()</title>
  </refsect2>
  <refsect2 id="{@id getppageid}">
   <title>getPPageId()</title>
  </refsect2>
 </refsect1>
 <refsect1 id="{@id utility}">
  <refsect2 id="{@id getsortedclasstreefromclass}">
   <title>getSortedClasstreeFromClass()</title>
  </refsect2>
  <refsect2 id="{@id hastutorial}">
   <title>hasTutorial()</title>
  </refsect2>
  <refsect2 id="{@id gettutorialtree}">
   <title>getTutorialTree()</title>
  </refsect2>
  <refsect2 id="{@id vardump-tree}">
   <title>vardump_tree()</title>
  </refsect2>
 </refsect1>
</refentry>
